"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from flask import json, request

from forms.accounts import AccountForm

from accountant import session_scope

from . import auth
from .. import api

from ..models.accounts import Account
from ..models.entries import Entry
from ..models.operations import Operation


@api.route("/accounts", methods=["GET"])
@auth.login_required
def get_accounts():
    """
    Returns accounts with their solds.
    """
    with session_scope() as session:
        query = Account.get_accounts(session)

        return json.dumps([{
            "id": i.id,
            "name": i.name,
            "authorized_overdraft": i.authorized_overdraft,
            "current": str(i.current),
            "pointed": str(i.pointed),
            "future": str(i.future)
        } for i in query.all()])


@api.route("/accounts/<account_id>/<year>/<month>/")
@auth.login_required
def get_account_status(account_id, year, month):
    with session_scope() as session:
        query = Operation.get_account_status(session, account_id, year, month)

        if query.count() == 1:
            result = query.one()
            revenues = result.revenues
            expenses = result.expenses
            balance = result.balance
        else:
            revenues = 0.0
            expenses = 0.0
            balance = 0.0

        return json.dumps({
            "expenses": str(expenses),
            "revenues": str(revenues),
            "balance": str(balance)
        })


@api.route("/accounts/<account_id>/months")
@auth.login_required
def get_months(account_id):
    with session_scope() as session:
        query = Entry.get_months_for_account(session, account_id)

        return json.dumps([{
            "year": i.year,
            "month": i.month.rjust(2, '0')
        } for i in query.all()])


@api.route("/accounts", methods=["PUT"])
@auth.login_required
def add_account():
    with session_scope() as session:
        account = Account(request.json['name'],
                          request.json['authorized_overdraft'])

        session.add(account)

        return json.dumps("Account added.")


@api.route("/accounts/<account_id>", methods=["PUT"])
@auth.login_required
def update_account(account_id):
    account_form = AccountForm()

    if account_form.validate():
        with session_scope() as session:
            account = session.query(Account).get(account_id)

            account.name = request.json['name']
            account.authorized_overdraft = request.json['authorized_overdraft']

            session.merge(account)

            return json.dumps("Account #%s updated." % account_id)
    else:
        return json.dumps({
            'ok': False,
            'error_type': 'validation',
            'errors': account_form.errorsi
        })


@api.route("/accounts/<account_id>", methods=["DELETE"])
@auth.login_required
def delete_account(account_id):
    with session_scope() as session:
        query = session.query(Account)
        query = query.filter(Account.id == account_id)

        account = query.first()

        session.delete(account)

        return json.dumps("Account #%s deleted." % account_id)
